"""Module for classification tasks."""
import pandas as pd
from sklearn.feature_extraction.text import TfidfVectorizer
from sklearn.feature_selection import chi2
import numpy as np
from sklearn.linear_model import LogisticRegression
from sklearn.ensemble import RandomForestClassifier
from sklearn.svm import LinearSVC
from sklearn.model_selection import cross_val_score
from sklearn.naive_bayes import MultinomialNB
from sklearn.metrics import confusion_matrix
import matplotlib.pyplot as plt
from sklearn.model_selection import train_test_split
import argparse
import sys

# from sklearn.ensemble import GradientBoostingClassifier
import seaborn as sns
from jarvis.db.figshare import data
from jarvis.db.jsonutils import dumpjson
import pickle
from sklearn.metrics import accuracy_score

parser = argparse.ArgumentParser(description="ChemNLP package.")
parser.add_argument(
    "--csv_path",
    default="filename.csv",
    help="Path for comma separated file.",
)
parser.add_argument(
    "--test_ratio",
    default=0.2,
    help="Test split ratio e.g. 0.2",
)

parser.add_argument(
    "--key_column",
    default="categories",
    help="Column name for classes in csv file",
)

parser.add_argument(
    "--value_column",
    default="title_abstract",
    help="Column name for text in csv file",
)

parser.add_argument(
    "--min_df",
    default=5,
    help="Minimum numbers of documents a word must be present in to be kept",
)
parser.add_argument(
    "--shuffle",
    default=False,
    help="Whether or not shuffle the rows in csv before splitting",
)


def generate_data(n_entries=None, nentry_each=None, filename="cond_mat.csv"):
    """Generate cond-mat dataset."""
    d = data("arXiv")
    df = pd.DataFrame(d)
    cond_mat_topics = [
        "cond-mat.mtrl-sci",
        "cond-mat.mes-hall",
        "cond-mat.str-el",
        "cond-mat.stat-mech",
        "cond-mat.supr-con",
        "cond-mat.soft",
        "cond-mat.quant-gas",
        "cond-mat.other",
        "cond-mat.dis-nn",
    ]
    df_cond_mat = df[
        df["categories"].apply(
            lambda x: "cond-mat" in x and x in cond_mat_topics
        )
    ]
    if n_entries is not None:
        df_cond_mat = df_cond_mat[0:n_entries]
    if nentry_each is not None:
        x = []
        for i in cond_mat_topics:
            x.extend(
                df_cond_mat[df_cond_mat["categories"] == i][0:nentry_each][
                    "id"
                ].values
            )
        df_cond_mat = df_cond_mat[df_cond_mat["id"].isin(x)]

    dff = df_cond_mat[["id", "title", "abstract", "categories"]]
    dff["title_abstract"] = dff["title"] + " " + df["abstract"]
    dff = dff.sample(frac=1, replace=True, random_state=1)
    dff.to_csv(filename)
    # print(len(set(dff.categories)))


def sk_class(
    csv_path=None,
    key="categories",
    value="title_abstract",
    test_size=0.2,
    min_df=5,
    ngram_range=[1, 2],
    print_common=False,
    model=None,
    categorize=True,  # False is buggy, need to check
    shuffle=False,
):
    """Classifcy data using scikit-learn library algos."""
    df = pd.read_csv(csv_path, dtype="str")
    # df=pd.read_csv(csv_path,dtype={'id':'str'})
    if categorize:
        df["category_id"] = df[key].factorize()[0]
        category_id_df = (
            df[[key, "category_id"]].sort_values("category_id")
            # df[[key, "category_id"]].drop_duplicates()
            # .sort_values("category_id")
        )

        category_to_id = dict(category_id_df.values)
        print("category_to_id", category_to_id)
    print(df)
    print(df[key].value_counts())
    title_tfidf = TfidfVectorizer(
        sublinear_tf=True,
        # logarithmic form for frequency
        min_df=min_df,
        # minimum numbers of documents a word must be present in to be kept
        norm="l2",
        # norm is set to l2, to ensure all our feature
        # vectors have a euclidian norm of 1
        encoding="latin-1",  # utf-8 possible
        ngram_range=ngram_range,
        # (1, 2) to indicate that we want to consider both unigrams and bigrams
        stop_words="english",
        # remove common pronouns ("a", "the", ...)
        # to reduce the number of noisy features
    )
    # Title features:
    title_features = title_tfidf.fit_transform(
        df[value]
    )  # .toarray()  # title
    if categorize:
        title_labels = df.category_id  # categories (cond-mat)
    else:
        title_labels = df[key]  # categories (cond-mat)
    # title_labels = df.category_id  # categories (cond-mat)
    print(
        "title_features.shape", title_features.shape
    )  # titles represented by features,
    # representing tf-idf score for different unigrams and bigrams
    if model is None:
        model = LogisticRegression(random_state=0)  # LinearSVC()
    (
        X_train,
        X_test,
        y_train,
        y_test,
        indices_train,
        indices_test,
    ) = train_test_split(
        title_features,
        title_labels,
        df.index,
        test_size=test_size,
        random_state=0,
        shuffle=shuffle,
    )
    if not categorize:
        category_to_id = list(set(df[key].values))
    id_to_category = dict((v, k) for k, v in category_to_id.items())
    info = {}
    train_info = {}
    test_info = {}
    for i, j in zip(df.id[indices_train], y_train):
        train_info[i] = id_to_category[int(j)]  # j
    for i, j in zip(df.id[indices_test], y_test):
        test_info[i] = id_to_category[int(j)]  # j
    info["train"] = train_info
    info["test"] = test_info
    dumpjson(data=info, filename="arXiv_categories.json")
    model.fit(X_train, y_train)
    y_pred = model.predict(X_test)
    f = open("pred_test.csv", "w")
    # f.write(str(accuracy_score(y_test, y_pred)))
    # f.close()
    line = "id,target,prediction\n"
    f.write(line)
    # pickle.dump(model, open("log.pk", "wb"))
    for i, j, k in zip(df.id[indices_test], y_test, y_pred):
        # print(i,j,k)
        line = (
            str(i)
            + ","
            + str(id_to_category[int(j)])
            + ","
            + str(id_to_category[int(k)])
            + "\n"
        )
        f.write(line)
    f.close()
    # print ('indices_train',df.id[indices_train]
    # .astype(str),len(indices_train))
    # print('indices_test',df.id[indices_test].astype(str),len(indices_test))
    print("Logistic model accuracy", accuracy_score(y_test, y_pred))
    plt.rcParams.update({"font.size": 20})
    conf_mat = confusion_matrix(y_test, y_pred)  # ,labels=category_to_id)
    fig, ax = plt.subplots(figsize=(16, 16))
    # print('category_id_df[key].values',category_id_df[key].values)
    sns.heatmap(
        conf_mat / conf_mat.sum(axis=1)[:, np.newaxis],
        annot=True,
        # fmt="d",
        fmt=".1%",
        cbar=False,
        square=True,
        cmap=sns.diverging_palette(20, 220, n=200),
        # xticklabels=category_id_df[k].values,
        # yticklabels=category_id_df[k].values,
        xticklabels=category_to_id,
        yticklabels=category_to_id,
    )
    plt.ylabel("Actual")
    plt.xlabel("Predicted")
    plt.tight_layout()
    plt.savefig("conf_log.pdf")
    plt.close()


def scikit_classify(
    df=None,
    key="term",
    value="title",
    CV=3,
    test_size=0.2,
    min_df=5,
    ngram_range=[1, 2],
    print_common=False,
    model_selection=False,
):
    """Classifcy data using scikit-learn library algos."""
    print(df)
    print(df[key].value_counts())
    df["category_id"] = df[key].factorize()[0]
    category_id_df = (
        df[[key, "category_id"]].drop_duplicates().sort_values("category_id")
    )

    category_to_id = dict(category_id_df.values)
    # id_to_category = dict(category_id_df[["category_id", key]].values)

    print(df.head())

    title_tfidf = TfidfVectorizer(
        sublinear_tf=True,
        # logarithmic form for frequency
        min_df=min_df,
        # minimum numbers of documents a word must be present in to be kept
        norm="l2",
        # norm is set to l2, to ensure all our feature
        # vectors have a euclidian norm of 1
        encoding="latin-1",  # utf-8 possible
        ngram_range=ngram_range,
        # (1, 2) to indicate that we want to consider both unigrams and bigrams
        stop_words="english",
        # remove common pronouns ("a", "the", ...)
        # to reduce the number of noisy features
    )

    # Title features:
    title_features = title_tfidf.fit_transform(
        df[value]
    )  # .toarray()  # title
    title_labels = df.category_id  # categories (cond-mat)
    print(
        "title_features.shape", title_features.shape
    )  # titles represented by features,
    # representing tf-idf score for different unigrams and bigrams

    if print_common:
        N = 2  # number of correlated items to display
        for title, category_id in sorted(category_to_id.items()):
            features_chi2 = chi2(title_features, title_labels == category_id)
            indices = np.argsort(features_chi2[0])
            feature_names = np.array(title_tfidf.get_feature_names_out())[
                indices
            ]
            unigrams = [v for v in feature_names if len(v.split(" ")) == 1]
            bigrams = [v for v in feature_names if len(v.split(" ")) == 2]
            print("# '{}':".format(title))
            print(
                "  . Most correlated unigrams:\n. {}".format(
                    "\n. ".join(unigrams[-N:])
                )
            )
            print(
                "  . Most correlated bigrams:\n. {}".format(
                    "\n. ".join(bigrams[-N:])
                )
            )
    if model_selection:
        models = [
            # GradientBoostingClassifier(),
            RandomForestClassifier(),
            # n_estimators=200, max_depth=3, random_state=0),
            LinearSVC(),
            MultinomialNB(),
            LogisticRegression(random_state=0),
            # try: multi_class='multinomial',
            # solver='lbfgs'...same accuracy actually
        ]

        cv_df = pd.DataFrame(index=range(CV * len(models)))
        entries = []
        for model in models:
            model_name = model.__class__.__name__
            accuracies = cross_val_score(
                model, title_features, title_labels, scoring="accuracy", cv=CV
            )  # features: ; labels:
            for fold_idx, accuracy in enumerate(accuracies):
                entries.append((model_name, fold_idx, accuracy))
        cv_df = pd.DataFrame(
            entries, columns=["model_name", "fold_idx", "accuracy"]
        )

        print(cv_df)
        # Visualize model accuracy with box plot and strip plot:

        plt.figure(figsize=(10, 8))
        plt.rcParams.update({"font.size": 18})
        sns.boxplot(x="model_name", y="accuracy", data=cv_df)
        sns.stripplot(
            x="model_name",
            y="accuracy",
            data=cv_df,
            size=8,
            jitter=True,
            edgecolor="gray",
            linewidth=2,
        )

        plt.ylim(0, 1)

        # plt.title('Model Benchmark Comparison from Title Text Data')
        plt.ylabel("Accuracy (cross-validation score)")
        # plt.xlabel('Model')
        plt.xlabel("")
        plt.xticks(rotation=45)
        plt.tight_layout()
        plt.savefig("cv.pdf")
        plt.close()
    print("Training logistic regression:")
    model = LogisticRegression(random_state=0)  # LinearSVC()
    (
        X_train,
        X_test,
        y_train,
        y_test,
        indices_train,
        indices_test,
    ) = train_test_split(
        title_features,
        title_labels,
        df.index,
        test_size=test_size,
        random_state=0,
    )
    model.fit(X_train, y_train)
    y_pred = model.predict(X_test)
    f = open("accuracy_logistic", "w")
    f.write(str(accuracy_score(y_test, y_pred)))
    f.close()
    print("Logistic", accuracy_score(y_test, y_pred))
    pickle.dump(model, open("log.pk", "wb"))

    plt.rcParams.update({"font.size": 20})
    conf_mat = confusion_matrix(y_test, y_pred)
    fig, ax = plt.subplots(figsize=(16, 16))
    sns.heatmap(
        conf_mat / conf_mat.sum(axis=1)[:, np.newaxis],
        annot=True,
        # fmt="d",
        fmt=".1%",
        cbar=False,
        square=True,
        cmap=sns.diverging_palette(20, 220, n=200),
        xticklabels=category_id_df[key].values,
        yticklabels=category_id_df[key].values,
    )
    plt.ylabel("Actual")
    plt.xlabel("Predicted")
    plt.tight_layout()
    plt.savefig("conf_log.pdf")
    plt.close()

    print("Training SVC:")
    model = LinearSVC()
    (
        X_train,
        X_test,
        y_train,
        y_test,
        indices_train,
        indices_test,
    ) = train_test_split(
        title_features,
        title_labels,
        df.index,
        test_size=test_size,
        random_state=0,
    )
    model.fit(X_train, y_train)
    y_pred = model.predict(X_test)
    print("SVC", accuracy_score(y_test, y_pred))
    pickle.dump(model, open("svc.pk", "wb"))

    plt.rcParams.update({"font.size": 20})
    conf_mat = confusion_matrix(y_test, y_pred)
    fig, ax = plt.subplots(figsize=(16, 16))
    sns.heatmap(
        conf_mat / conf_mat.sum(axis=1)[:, np.newaxis],
        annot=True,
        # fmt="d",
        fmt=".1%",
        cbar=False,
        square=True,
        cmap=sns.diverging_palette(20, 220, n=200),
        xticklabels=category_id_df[key].values,
        yticklabels=category_id_df[key].values,
    )
    plt.ylabel("Actual")
    plt.xlabel("Predicted")
    plt.tight_layout()
    plt.savefig("conf_svc.pdf")
    plt.close()
    f = open("accuracy_svc", "w")
    f.write(str(accuracy_score(y_test, y_pred)))
    f.close()


if __name__ == "__main__":
    # python generate_data.py
    # generate_data()
    # sk_class(csv_path='cond_mat.csv')
    args = parser.parse_args(sys.argv[1:])
    csv_path = args.csv_path
    test_ratio = float(args.test_ratio)
    key_column = args.key_column
    value_column = args.value_column
    min_df = int(args.min_df)
    shuffle = args.shuffle

    sk_class(
        csv_path=csv_path,
        test_size=test_ratio,
        key=key_column,
        value=value_column,
        min_df=min_df,
        shuffle=shuffle,
    )
    # df = pd.read_csv("cond_mat.csv")
    # scikit_classify(df=df, key="categories", value="title_abstract")



"""Module to perform BERT classification."""
from transformers import AdamW, get_linear_schedule_with_warmup
import random
import numpy as np
import torch
from tqdm import tqdm
import pandas as pd
from transformers import BertTokenizer
from torch.utils.data import TensorDataset
from transformers import BertForSequenceClassification
from torch.utils.data import DataLoader, RandomSampler, SequentialSampler
from sklearn.metrics import f1_score
from sklearn.model_selection import train_test_split
from sklearn.metrics import accuracy_score

# from tqdm import tqdm
# from tqdm.notebook import tqdm
# from torchmetrics import Accuracy
# from jarvis.db.figshare import data
# import matplotlib.pyplot as plt

# df = df_cond_mat_b[0:2000]
# key = "term"
# value = "title"
# batch_size = 32
# epochs = 5

# os.environ["CUDA_LAUNCH_BLOCKING"] = "1"
# !export CUDA_LAUNCH_BLOCKING="1"


def train_classifier(
    df=None,
    key="term",
    value="title",
    batch_size=32,
    epochs=5,
    seed_val=17,
    test_size=0.2,
    random_state=0,
    max_length=128,
    lr=0.001,
    model_save_frequency=3,
):
    """Train a PyTorch BERT classifier."""
    label_dict = {}
    possible_labels = df[key].unique()
    for index, possible_label in enumerate(possible_labels):
        label_dict[possible_label] = index
    print("label_dict", label_dict)
    df["label"] = df[key].replace(label_dict)
    X_train, X_val, y_train, y_val = train_test_split(
        df.index.values,
        df.label.values,
        test_size=test_size,
        random_state=random_state,
        stratify=df.label.values,
    )
    print("X_train", X_train)
    df["data_type"] = ["not_set"] * df.shape[0]

    df.loc[X_train, "data_type"] = "train"
    df.loc[X_val, "data_type"] = "val"

    tokenizer = BertTokenizer.from_pretrained(
        "bert-base-uncased", do_lower_case=True
    )

    encoded_data_train = tokenizer.batch_encode_plus(
        df[df.data_type == "train"][value].values,
        add_special_tokens=True,
        return_attention_mask=True,
        pad_to_max_length=True,
        max_length=max_length,
        return_tensors="pt",
    )

    encoded_data_val = tokenizer.batch_encode_plus(
        df[df.data_type == "val"][value].values,
        add_special_tokens=True,
        return_attention_mask=True,
        pad_to_max_length=True,
        max_length=max_length,
        return_tensors="pt",
    )
    input_ids_train = encoded_data_train["input_ids"]
    attention_masks_train = encoded_data_train["attention_mask"]
    labels_train = torch.tensor(df[df.data_type == "train"].label.values)

    input_ids_val = encoded_data_val["input_ids"]
    attention_masks_val = encoded_data_val["attention_mask"]
    labels_val = torch.tensor(df[df.data_type == "val"].label.values)

    dataset_train = TensorDataset(
        input_ids_train, attention_masks_train, labels_train
    )
    dataset_val = TensorDataset(input_ids_val, attention_masks_val, labels_val)
    model = BertForSequenceClassification.from_pretrained(
        "bert-base-uncased",
        num_labels=len(label_dict),
        output_attentions=False,
        output_hidden_states=False,
    )

    dataloader_train = DataLoader(
        dataset_train,
        sampler=RandomSampler(dataset_train),
        batch_size=batch_size,
    )

    dataloader_validation = DataLoader(
        dataset_val,
        sampler=SequentialSampler(dataset_val),
        batch_size=batch_size,
    )

    optimizer = AdamW(model.parameters(), lr=lr, eps=1e-8)

    scheduler = get_linear_schedule_with_warmup(
        optimizer,
        num_warmup_steps=0,
        num_training_steps=len(dataloader_train) * epochs,
    )

    def f1_score_func(preds, labels):
        preds_flat = np.argmax(preds, axis=1).flatten()
        labels_flat = labels.flatten()
        return f1_score(labels_flat, preds_flat, average="weighted")

    def torch_accuracy_score(preds, labels):
        preds_flat = np.argmax(preds, axis=1).flatten()
        labels_flat = labels.flatten()
        return accuracy_score(labels_flat, preds_flat)

    def accuracy_per_class(preds, labels):
        label_dict_inverse = {v: k for k, v in label_dict.items()}

        preds_flat = np.argmax(preds, axis=1).flatten()
        labels_flat = labels.flatten()

        for label in np.unique(labels_flat):
            y_preds = preds_flat[labels_flat == label]
            y_true = labels_flat[labels_flat == label]
            print(f"Class: {label_dict_inverse[label]}")
            print(f"Accuracy: {len(y_preds[y_preds==label])}/{len(y_true)}\n")

    # seed_val = 17
    # torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
    # device = "cpu"
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
    random.seed(seed_val)
    np.random.seed(seed_val)
    torch.manual_seed(seed_val)
    torch.cuda.manual_seed_all(seed_val)

    def evaluate(dataloader_val):
        model.to(device)
        model.eval()

        loss_val_total = 0
        predictions, true_vals = [], []

        for batch in dataloader_val:

            batch = tuple(b.to(device) for b in batch)

            inputs = {
                "input_ids": batch[0],
                "attention_mask": batch[1],
                "labels": batch[2],
            }

            with torch.no_grad():
                outputs = model(**inputs)

            loss = outputs[0]
            logits = outputs[1]
            loss_val_total += loss.item()

            logits = logits.detach().cpu().numpy()
            label_ids = inputs["labels"].cpu().numpy()
            predictions.append(logits)
            true_vals.append(label_ids)

        loss_val_avg = loss_val_total / len(dataloader_val)

        predictions = np.concatenate(predictions, axis=0)
        true_vals = np.concatenate(true_vals, axis=0)

        return loss_val_avg, predictions, true_vals

    for epoch in tqdm(range(1, epochs + 1)):
        model.to(device)
        model.train()

        loss_train_total = 0

        progress_bar = tqdm(
            dataloader_train,
            desc="Epoch {:1d}".format(epoch),
            leave=False,
            disable=False,
        )
        for batch in progress_bar:

            model.zero_grad()

            batch = tuple(b.to(device) for b in batch)

            inputs = {
                "input_ids": batch[0],
                "attention_mask": batch[1],
                "labels": batch[2],
            }

            outputs = model(**inputs)

            loss = outputs[0]
            loss_train_total += loss.item()
            loss.backward()

            torch.nn.utils.clip_grad_norm_(model.parameters(), 1.0)

            optimizer.step()
            scheduler.step()

            progress_bar.set_postfix(
                {"training_loss": "{:.3f}".format(loss.item() / len(batch))}
            )

        if epoch == epochs:
            torch.save(
                model.state_dict(), f"finetuned_BERT_epoch_{epoch}.model"
            )

        tqdm.write(f"\nEpoch {epoch}")

        loss_train_avg = loss_train_total / len(dataloader_train)
        tqdm.write(f"Training loss: {loss_train_avg}")

        val_loss, predictions, true_vals = evaluate(dataloader_validation)
        val_f1 = f1_score_func(predictions, true_vals)
        val_acc = torch_accuracy_score(predictions, true_vals)
        tqdm.write(f"Validation loss: {val_loss}")
        tqdm.write(f"F1 Score (Weighted): {val_f1}")
        tqdm.write(f"Accuracy: {val_acc}")
        # accuracy_per_class(predictions, true_vals)


if __name__ == "__main__":
    df = pd.read_csv("cond_mat.csv")

    train_classifier(df=df)



import argparse
# import os
from chemnlp.utils.process_doc import ProcessDoc
import sys

parser = argparse.ArgumentParser(
    description="Natural language processing for chemical data."
)

parser.add_argument(
    "--file_path",
    default="./",
    help="Filepath for text file parsing.",
)

if __name__ == "__main__":
    args = parser.parse_args(sys.argv[1:])
    with open(args.file_path, "r") as f:
        txt = f.read()  # .splitlines()
    print("txt", txt)
    pdoc = ProcessDoc(text=txt)
    print(pdoc.get_chemical_formulae())



"""Module for some named entities."""

# Dictionaries with keywords for research areas:


def keywords():
    """Get named entities."""
    keys = {
        "solarMaterials": [
            "solar",
            "power conversion efficiency",
            "pce",
            "schottky-queisser limit",
            "short circuit current",
            "isc",
            "voc",
            "fill factor",
            "diffusion length",
            "transmission",
            "scatter",
            "photon",
            "renewable",
            "photovoltaic",
            "infrared",
            "lifetime",
            "diffusion coefficient",
            "impedance spectroscopy",
            "optical interference",
            "schottky",
            "junction",
            "ohmic",
            "heterojunction",
            "homojunction",
            "band alignment",
            "current density",
            "solar simulator",
            "spectral irradiance",
            "conductivity",
            "resistivity",
            "sheet resistance",
            "open circuit voltage",
            "absorption coefficient",
            "tandem solar cell",
            "drift",
            "band bending",
        ],
        "magneticMaterials": [
            "magnetic",
            "hysteresis",
            "curie",
            "magneton",
            "coercive field",
            "susceptibility",
        ],
        "batteryMaterials": [
            "cathode",
            "anode",
            "electrode",
            "electrolyte",
            "capacity",
            "energy density",
            "separator",
            "ionic conductivity",
            "electric vehicle",
            "battery",
            "energy storage",
            "nernst",
            "faraday",
            "faradaic charge transfer",
            "pseudocapacitor",
            "pseudocapacitance",
            "surface-limited",
            "electrochemistry",
            "electrochemical",
            "supercapacitor",
            "ion mobility",
            "diffusion coefficient",
            "ac impedance",
            "electrochemical impedance spectroscopy",
            "overpotential",
            "reduction potential",
            "oxidation potential",
            "redox",
            "current density",
            "cycle number",
            "cycle life",
            "discharge cycle",
            "cycle lifetime",
            "diffusion limited",
            "dendrite",
        ],
        "2DMaterials": [
            "interlayer excitions",
            "exciton binding energy",
            "flexible",
            "vdw",
            "exfoliate",
            "cvd",
            "mixed-dimensional heterostructure",
            "excitonics",
            "valleytronics",
            "flexible electronics",
            "tmd",
            "chalcogenide",
            "mxene",
            "xene",
            "mof",
            "perovskite",
            "cof",
            "spintronics",
            "cswap",
            "vth",
            "electrostatic doping",
            "defect passivation",
            "metal organic framework",
            "moire superlattice",
            "bilayer",
            "trilayer",
            "monolayer",
            "mechanical exfoliation",
            "trion",
            "van der waals",
            "wearable",
            "graphene",
            "threshold voltage",
            "covalent organic framework",
            "polariton",
            "valley degree of freedom",
        ],
        "photocatalystMaterials": [
            "charge transfer",
            "inorganic coordination complex",
            "transition metal",
            "co2 reduction",
            "water splitting",
            "n2 fixation",
            "charge separation",
        ],
    }
    return keys



"""Module to parse documents."""

from jarvis.core.composition import Composition
from chemdataextractor import Document
import math


class ProcessDoc(object):
    """Module to parse documents."""

    def __init__(self, doc_id=None, text=""):
        """Initialize the class."""
        self.text = text
        self.doc_id = doc_id

    def get_chemical_formulae(self):
        """Get chemical formulae."""
        formulae = []
        doc = Document(self.text)
        for j in doc.cems:
            wt = "na"
            try:
                form = Composition.from_string(j.text)
                wt = float(form.weight)
            except Exception:
                pass
            if (
                wt != "na"
                and not math.isnan(wt)
                # and form.nspecies > 1
                # and "cond-mat" in i["categories"]
            ):
                # print(i['id'],i['title'],j,form.reduced_formula)

                if form.reduced_formula not in formulae:
                    formulae.append(form.reduced_formula)
        return formulae


if __name__ == "__main__":
    txt = "MgB2 is a superconductor."
    pdoc = ProcessDoc(text=txt)
    print(pdoc.get_chemical_formulae())



"""Module to generate word-cloud."""

import os
import matplotlib.pyplot as plt
from os import path
from wordcloud import WordCloud
import pandas as pd


def make_plot(df=None, key="title"):
    """Generate a word-cloud given pandas dataframe."""
    all_text = " ".join(df[key].values)

    f = open("sample.txt", "w")
    f.write(all_text)
    f.close()
    d = path.dirname(__file__) if "__file__" in locals() else os.getcwd()

    # Read the whole text.
    text = open(path.join(d, "sample.txt")).read()

    # Generate a word cloud image
    wordcloud = WordCloud().generate(text)

    # Display the generated image:
    # the matplotlib way:
    # plt.imshow(wordcloud, interpolation='bilinear')
    # plt.axis("off")

    # lower max_font_size
    wordcloud = WordCloud(max_font_size=40, background_color="white").generate(
        text
    )
    plt.figure()
    plt.imshow(wordcloud, interpolation="bilinear")
    plt.axis("off")
    plt.savefig("wordc.png")
    plt.close()


if __name__ == "__main__":
    df = pd.read_csv("cond_mat.csv")

    key = "title"
    make_plot(df=df, key=key)



"""Module to perform tsne given csv file."""
import matplotlib.pyplot as plt

import pandas as pd

from tqdm import tqdm
from collections import Counter
from nltk.stem import PorterStemmer
from nltk.tokenize import word_tokenize
import numpy as np
from sklearn.decomposition import TruncatedSVD
from sklearn.manifold import TSNE
from time import time
import nltk

nltk.download("punkt")
# samples=len(df)
# import numpy as np
# import os
# from jarvis.db.figshare import data
# import requests
# import matplotlib.colors
# import matplotlib.cm as cm
# import feedparser
# import time
# import json


def abstract2df(df=None, idx=None, text=""):
    ps = PorterStemmer()
    """
    USAGE: Use Porter Stemming to process a paper abstract.

    ARGUMENT:
    df - DataFrame with keys ["term", "title", "summary", "id"]
    idx - index locating paper within DataFrame

    RETURNS: stem_df - DataFrame with keys ["stems", "counts", "docs"]
    """
    abstract = df.iloc[idx][text]
    # abstract = df.iloc[idx].summary
    words = word_tokenize(abstract)
    stems = [ps.stem(w).lower() for w in words]
    counts = Counter(stems)
    docs = list(np.ones(len(counts)).astype(int))
    stem_df = pd.DataFrame(
        {
            "stems": list(counts.keys()),
            "counts": list(counts.values()),
            "docs": docs,
        }
    )
    stem_df = stem_df[["stems", "counts", "docs"]]
    return stem_df


def docs2idf(df, num):
    """
    USAGE:
    Get an inverse document frequency (idf) measure for each stem,
    and add that measure to the word stem DataFrame.

    ARGUMENTS:
    df - word stem DataFrame
    num - total number of papers in document pool
    """
    df["idf"] = np.log(num / df["docs"])
    # print ('df',df)
    return df


def reduce_dim(matrix, dims):
    """
    USAGE: Perform Truncated SVD to reduce dimensionality of embedding space.

    ARGUMENTS:
    matrix - training data (sparse matrix) with shape (n_features, n_samples)
    dims - number of dimensions to project data into

    RETURNS:
    X_new - array of reduced dimensionality
    """

    tic = time()
    print("Performing Truncated SVD...")
    svd = TruncatedSVD(n_components=dims, random_state=1)
    X = svd.fit_transform(matrix)  # matrix.shape = (n_samples, n_features)
    toc = time()
    print(
        "Embeddings reduced from %d to %d using TruncatedSVD. (Time: %.2f s)"
        % (matrix.shape[1], dims, (toc - tic))
    )
    return X


def TSNE2D(X):
    """
    USAGE: Perform TSNE to reduce embedding space to 2D
    ARGUMENT: X - high-dimensional training array (n_samples, n_features ~ 100)
    RETURNS: X_embedded - 2D matrix (n_samples, 2)
    """

    tic = time()
    print("Performing TSNE...")
    X_embedded = TSNE(
        n_components=2, perplexity=30, random_state=1
    ).fit_transform(X)
    toc = time()
    print(
        "Embeddings reduced to 2 dimensions through TSNE. (Time: %.2f s)"
        % (toc - tic)
    )
    return X_embedded


def tsne(df=None, category_key="term", text="title", filename=None, ndim=128):
    print("Counting word stems in %d abstracts..." % len(df))
    samples = len(df)
    full_stem_df = pd.DataFrame()
    for idx in range(len(df)):
        # print(idx)
        stem_df = abstract2df(df=df, idx=idx, text=text)
        full_stem_df = pd.concat([full_stem_df, stem_df])
        if idx and idx % 10 == 0 or idx == len(df) - 1:
            full_stem_df = full_stem_df.groupby("stems").sum()
            full_stem_df["stems"] = full_stem_df.index
            full_stem_df = full_stem_df[["stems", "counts", "docs"]]
    full_stem_df.sort_values("counts", ascending=False, inplace=True)
    all_stems = full_stem_df  # count_stems(dfb)
    all_stems = docs2idf(all_stems, len(df))
    all_stems.reset_index(drop=True, inplace=True)
    # df=dfb
    stems = all_stems
    # assert samples <= len(df)
    idf = np.array(list(stems["idf"]))
    stems = stems[["stems", "idf"]]
    matrix = np.zeros((samples, len(stems)))

    info = []
    print("Embedding paper abstracts...")
    i = 0
    for idx in tqdm(range(samples)):
        # print(idx)
        information = list(df.iloc[idx][["id", category_key, text]])
        info.append(information)
        new_df = pd.merge(
            stems,
            abstract2df(df=df, idx=idx, text=text),
            on="stems",
            how="left",
        ).fillna(0)
        vec = np.array(list(new_df["counts"]))
        vec /= np.sum(vec)  # components sum to 1
        vec *= idf  # apply inverse document frequency
        matrix[i, :] = vec
        i += 1
    print("Paper abstracts converted to vectors.")

    matrix = np.nan_to_num(matrix, nan=0.0)
    X = reduce_dim(matrix, ndim)
    X_embedded = TSNE2D(X)

    # %matplotlib inline
    plt.rcParams.update({"font.size": 14})
    plt.figure(figsize=(8, 8))
    X = X_embedded
    x = X[:, 0]
    y = X[:, 1]

    term_list = list(np.array(info).T[1])
    term_set = list(set(term_list))
    term_list = [term_set.index(term) for term in term_list]

    color_list = plt.cm.tab10(term_list)

    lbls = []
    xyz = []
    for i, j, k, p in zip(x, y, term_list, color_list):
        if k not in lbls:
            lbls.append(k)
            xyz.append([i, j, k])
            plt.scatter(i, j, s=10, c=p, label=term_set[k])

    plt.scatter(x, y, s=10, c=color_list)  # ,label=term_list)
    plt.legend(loc="lower left")
    # plt.xlim([-40,40])
    # plt.ylim([-40,40])
    plt.xticks([])
    plt.yticks([])
    if filename is not None:
        plt.savefig(filename)
        plt.close()
    plt.show()


# plot_2D(X_embedded, info)
# plt.savefig('tsne.png')
# plt.close()

if __name__ == "__main__":
    df = pd.read_csv("id_term_title.csv")
    tsne(df=df, filename="x.png")



"""Version number."""
__version__ = "2023.3.4"

import os


def test(*args):
    """Run pytest in the base of chemnlp."""
    import pytest

    path = os.path.join(os.path.split(__file__)[0], "tests")
    pytest.main(args=[path] + list(args))
